package dyswis;

//A singleton class that has the interface related properties
public class WLANInterface implements java.io.Serializable {
	/**
	 * 
	 */
	private static WLANInterface wlan = null;
	private static final long serialVersionUID = 1L;
	//Assuming values are populated every 5 mins in an hour
	private int TIME_INTERVAL_SLOTS = 12;
	
	String wlanInterfaceName;
	//Interface ID as stored by the OS
	String wlanInterfaceID;
	//Contains the number of successfully transmitted MSDU/MMPDUs.
	String transmittedFrameCount;
	//Contains the number of MSDU/MMPDUs successfully received
	String receivedFrameCount;
	
	//Contains the number of times an expected ACK 
	//has not been received.
	String ackFailureCount;
	
	//Contains the number of times an FCS error has been 
	//detected in a received MPDU.
	String fcsErrorCount;
	
	//A percentage value that represents the signal 
	//quality of the network
	long [] wlanSignalQuality;
	
	//Stores the number of bytes transmitted for the past 10000 seconds
	long [] bytesExchanged;
	
	//This field is used to maintain the above arrays as a circular queue
	private int slotsFilled;
	
	double meanSignalStrength;
	double signalStandardDeviation;
	
	public double getMeanSignalStrength() {
		return meanSignalStrength;
	}
	public void setMeanSignalStrength(double meanSignalStrength) {
		this.meanSignalStrength = meanSignalStrength;
	}

	private WLANInterface(){
		//Initializing the signal quality, and bytes exchanged arrays
		wlanSignalQuality = new long[100];
		bytesExchanged = new long[100];
		slotsFilled = 0;
	}
	
	//Implementing the singleton logic
	public static WLANInterface getWLANInterfaceObject(){
		if(wlan == null){
			wlan = new WLANInterface();
		}
		return wlan;
	}
	
	public double calculateStandardDeviation(long [] input){
		int sum = 0;
		long meanvar = 0;
		int mean = 0;
		double sd = 0;
		for(int i = 0; i < input.length; i++){
			sum += input[i]; 
		}
		mean = sum/input.length;
		this.setMeanSignalStrength(mean);
		for (int i = 0; i < input.length; i++) {
			meanvar += (mean - input[i]) * (mean - input[i]);
		}
		sd = Math.sqrt(meanvar/input.length);
		return sd;
	}
	public double getSignalStandardDeviation() {
		return signalStandardDeviation;
	}
	public void setSignalStandardDeviation(double signalStandardDeviation) {
		this.signalStandardDeviation = signalStandardDeviation;
	}
	public void increaseSlotsFilled(){
		this.slotsFilled = (this.slotsFilled+1)%TIME_INTERVAL_SLOTS;
	}
	public String getWlanInterfaceName() {
		return wlanInterfaceName;
	}

	public void setWlanInterfaceName(String wlanInterfaceName) {
		this.wlanInterfaceName = wlanInterfaceName;
	}

	public String getWlanInterfaceID() {
		return wlanInterfaceID;
	}

	public void setWlanInterfaceID(String wlanInterfaceID) {
		this.wlanInterfaceID = wlanInterfaceID;
	}

	public String getTransmittedFrameCount() {
		return transmittedFrameCount;
	}

	public void setTransmittedFrameCount(String transmittedFrameCount) {
		this.transmittedFrameCount = transmittedFrameCount;
	}

	public String getReceivedFrameCount() {
		return receivedFrameCount;
	}

	public void setReceivedFrameCount(String receivedFrameCount) {
		this.receivedFrameCount = receivedFrameCount;
	}

	public String getAckFailureCount() {
		return ackFailureCount;
	}

	public void setAckFailureCount(String ackFailureCount) {
		this.ackFailureCount = ackFailureCount;
	}

	public String getFcsErrorCount() {
		return fcsErrorCount;
	}

	public void setFcsErrorCount(String fcsErrorCount) {
		this.fcsErrorCount = fcsErrorCount;
	}

	public long[] getWlanSignalQuality() {
		return wlanSignalQuality;
	}

	/*******************************************************************************
	 * setWlanSignalQuality() and setBytesExchanged() operate on wlanSignalQuality *
	 * and  bytesExchanged as a circular queue. setWlanSignalQuality() is fired    *
	 * every 10 seconds and thus each slot in the array is filled after 10 seconds *
	 * and updates the slotsFilled field. setBytesExchanged() is called when a     *
	 * packet is captured and since slotsFilled is updated after 10 seconds only,  *
	 * it appears as if each slot in the bytesExchanged array holds bytes          * 
	 * exchanged every 10 seconds                                                  *
	 * *****************************************************************************
	 */
	public void setWlanSignalQuality(long wlanSignalQuality) {
		this.wlanSignalQuality[this.slotsFilled] = wlanSignalQuality;
		this.setSignalStandardDeviation(this.calculateStandardDeviation(this.wlanSignalQuality));
		this.slotsFilled = (this.slotsFilled+1)%100;
	}
	public long[] getBytesExchanged() {
		return bytesExchanged;
	}
	public void setBytesExchanged(long bytesExchanged) {
		this.bytesExchanged[this.slotsFilled] += bytesExchanged;
	}
	
};
